@@ -1,12 +1,12 @@
 /*-------------------------------------------------------------------------
  *
- *	  WIN1256 and UTF-8
+ *	  WIN1256 and UTF8
  *
- * Portions Copyright (c) 1996-2003, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2005, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql-server/src/backend/utils/mb/conversion_procs/utf8_and_win1256/utf8_and_win1256.c,v 1.6 2003/08/04 02:40:08 momjian Exp $
+ *	  $PostgreSQL: pgsql/src/backend/utils/mb/conversion_procs/utf8_and_win1256/utf8_and_win1256.c,v 1.13 2005/10/15 02:49:35 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -17,11 +17,11 @@
 #include "../../Unicode/utf8_to_win1256.map"
 #include "../../Unicode/win1256_to_utf8.map"
 
-PG_FUNCTION_INFO_V1(utf_to_win1256);
-PG_FUNCTION_INFO_V1(win1256_to_utf);
+PG_FUNCTION_INFO_V1(utf8_to_win1256);
+PG_FUNCTION_INFO_V1(win1256_to_utf8);
 
-extern Datum utf_to_win1256(PG_FUNCTION_ARGS);
-extern Datum win1256_to_utf(PG_FUNCTION_ARGS);
+extern Datum utf8_to_win1256(PG_FUNCTION_ARGS);
+extern Datum win1256_to_utf8(PG_FUNCTION_ARGS);
 
 /* ----------
  * conv_proc(
@@ -35,10 +35,10 @@ extern Datum win1256_to_utf(PG_FUNCTION_ARGS);
  */
 
 Datum
-utf_to_win1256(PG_FUNCTION_ARGS)
+utf8_to_win1256(PG_FUNCTION_ARGS)
 {
-	unsigned char *src = PG_GETARG_CSTRING(2);
-	unsigned char *dest = PG_GETARG_CSTRING(3);
+	unsigned char *src = (unsigned char *) PG_GETARG_CSTRING(2);
+	unsigned char *dest = (unsigned char *) PG_GETARG_CSTRING(3);
 	int			len = PG_GETARG_INT32(4);
 
 	Assert(PG_GETARG_INT32(0) == PG_UTF8);
@@ -52,10 +52,10 @@ utf_to_win1256(PG_FUNCTION_ARGS)
 }
 
 Datum
-win1256_to_utf(PG_FUNCTION_ARGS)
+win1256_to_utf8(PG_FUNCTION_ARGS)
 {
-	unsigned char *src = PG_GETARG_CSTRING(2);
-	unsigned char *dest = PG_GETARG_CSTRING(3);
+	unsigned char *src = (unsigned char *) PG_GETARG_CSTRING(2);
+	unsigned char *dest = (unsigned char *) PG_GETARG_CSTRING(3);
 	int			len = PG_GETARG_INT32(4);
 
 	Assert(PG_GETARG_INT32(0) == PG_WIN1256);
@@ -63,7 +63,7 @@ win1256_to_utf(PG_FUNCTION_ARGS)
 	Assert(len >= 0);
 
 	LocalToUtf(src, dest, LUmapWIN1256,
-		sizeof(LUmapWIN1256) / sizeof(pg_local_to_utf), PG_WIN1256, len);
+			sizeof(LUmapWIN1256) / sizeof(pg_local_to_utf), PG_WIN1256, len);
 
 	PG_RETURN_VOID();
 }
